﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace NugetRestClient
{
    class NugetClient
    {
        const string _sourceUrl = "https://api.nuget.org/v3/index.json";
        CookieContainer _cookies;
        HttpClient _httpClient;
        ServiceIndex _serviceIndex;

        internal async Task<IEnumerable<PackageInfo>> GetAllPackageListAsync(int maxResult = 100)
        {
            string url = GetServiceEndPoint("SearchQueryService");
            string query = string.Format("{0}", url);

            List<PackageInfo> list = new List<PackageInfo>();

            int currentPosition = 0;
            int pageSize = 50;

            while (true)
            {
                int nextPage = Math.Min(maxResult, pageSize);
                maxResult -= pageSize;

                string nextQuery = query + string.Format("?skip={0}&take={1}", currentPosition, currentPosition + nextPage);

                string text = await _httpClient.GetStringAsync(nextQuery);
                SearchQueryServiceResult searchResult = Newtonsoft.Json.JsonConvert.DeserializeObject<SearchQueryServiceResult>(text);

                list.AddRange(searchResult.data);

                currentPosition += nextPage;
                if (maxResult <= 0)
                {
                    break;
                }
            }

            return list;
        }

        public async Task<PackageMetadata> GetPackageMetadataAsync(string packageId, string vesrionPostfix)
        {
            string url = GetServiceEndPoint("RegistrationsBaseUrl");
            string query = string.Format("{0}{1}/{2}.json", url, packageId.ToLower(), vesrionPostfix);

            HttpResponseMessage hrm = await _httpClient.GetAsync(query);

            if (hrm.StatusCode == HttpStatusCode.NotFound)
            {
                return null;
            }

            string text = await hrm.Content.ReadAsStringAsync();
            PackageMetadata searchResult = Newtonsoft.Json.JsonConvert.DeserializeObject<PackageMetadata>(text);
            return searchResult;
        }

        public async Task<bool> UnlistVersionPackageAsync(string packageId, string vesrionPostfix)
        {
            string url = GetServiceEndPoint("PackagePublish/2.0.0");
            string query = string.Format("{0}/{1}/{2}", url, packageId.ToLower(), vesrionPostfix);

            HttpResponseMessage hrm = await _httpClient.DeleteAsync(query);
            return hrm.StatusCode == HttpStatusCode.OK;
        }

        public async Task<PackageInfo> GetPackageInfoAsync(string packageId, bool includePrerelease)
        {
            string url = GetServiceEndPoint("SearchQueryService");
            string query = string.Format("{0}?q={1}", url, packageId);

            if (includePrerelease == true)
            {
                query += "&prerelease=true";
            }

            string text = await _httpClient.GetStringAsync(query);
            SearchQueryServiceResult searchResult = Newtonsoft.Json.JsonConvert.DeserializeObject<SearchQueryServiceResult>(text);
            if (searchResult.totalHits == 0)
            {
                return null;
            }

            return searchResult.data[0];
        }

        private string GetServiceEndPoint(string serviceTypeName)
        {
            foreach (var item in _serviceIndex.resources)
            {
                if (item.type == serviceTypeName)
                {
                    return item.id;
                }
            }

            throw new ApplicationException("ServiceNotFound: " + serviceTypeName);
        }

        public NugetClient(string apiKey = "")
        {
            HttpClientHandler handler = new HttpClientHandler();
            _cookies = new CookieContainer();
            handler.CookieContainer = _cookies;

            HttpClient hc = new HttpClient(handler);
            _httpClient = hc;

            if (string.IsNullOrEmpty(apiKey) == false)
            {
                _httpClient.DefaultRequestHeaders.Add("X-NuGet-ApiKey", apiKey);
            }
        }

        public async Task GetFeedAsync()
        {
            string text = await _httpClient.GetStringAsync(_sourceUrl);
            _serviceIndex = Newtonsoft.Json.JsonConvert.DeserializeObject<ServiceIndex>(text);
        }
    }
}
